#!/usr/bin/perl
#
# Sensor for gathering hardware data for Zabbix via IPMI.
# Copyright (c) 2008 Jakov Sosic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Changes and Modifications
# * Thu May 15 03:40:39 CEST 2008       
#       - script created
#

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $ipmitool="/usr/bin/ipmitool";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd";

# signal handling
$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
};

local $SIG{TERM} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
};

# fetch status from mod_status
alarm 20;
my $cmd = "sudo $ipmitool sdr list";
my $server_status = qx($cmd);

# get information from status output
alarm 3;
my($ChassisTemp, $Temp1,$Temp2,$Temp3,$Temp4,$Temp5,$Temp6,$Temp7,$Temp8,$Temp9, $Fan1,$Fan2,$Fan3,$Fan4, $Fan5, $Fan6);

$Temp1 = $1 if ($server_status =~ s/Temp 1\s+\|\s+([\d]+)\s+degrees//);
$Temp2 = $1 if ($server_status =~ s/Temp 2\s+\|\s+([\d]+)\s+degrees//);
$Temp3 = $1 if ($server_status =~ s/Temp 3\s+\|\s+([\d]+)\s+degrees//);
$Temp4 = $1 if ($server_status =~ s/Temp 4\s+\|\s+([\d]+)\s+degrees//);
$Temp5 = $1 if ($server_status =~ s/Temp 5\s+\|\s+([\d]+)\s+degrees//);
$Temp6 = $1 if ($server_status =~ s/Temp 6\s+\|\s+([\d]+)\s+degrees//);
$Temp7 = $1 if ($server_status =~ s/Temp 7\s+\|\s+([\d]+)\s+degrees//);
$Temp8 = $1 if ($server_status =~ s/Temp 8\s+\|\s+([\d]+)\s+degrees//);
$Temp9 = $1 if ($server_status =~ s/Temp 9\s+\|\s+([\d]+)\s+degrees//);

$Fan1 = $1 if ($server_status =~ /Fan 1 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan2 = $1 if ($server_status =~ /Fan 2 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan3 = $1 if ($server_status =~ /Fan 3 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan4 = $1 if ($server_status =~ /Fan 4 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan5 = $1 if ($server_status =~ /Fan 5 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan6 = $1 if ($server_status =~ /Fan 6 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
#print $Temp1."--\n";


# send information to Zabbix server
alarm 10;
if ( qx($zabbix_send_command -k ipmi.temp.1 -o $Temp1) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.2 -o $Temp2) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.3 -o $Temp3) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.4 -o $Temp4) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.5 -o $Temp5) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.6 -o $Temp6) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.7 -o $Temp7) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.8 -o $Temp8) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.9 -o $Temp9) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.1 -o $Fan1) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.2 -o $Fan2) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.3 -o $Fan3) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.4 -o $Fan4) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.5 -o $Fan5) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.6 -o $Fan6) =~ /failed:\ 1/) { print"0\n"; exit(-1);}
print "1\n";
exit(0);
# EOF


